% Nonlinear ODE Model of the toehold-mediated DSD time delay circuit
% Input: Initial conditions of S, I, R, and the desired time delay "t_delay"
% Output: D - the initial concentration of "Delay" complex

close all        % Close all open figures
clear            % Clear the workspace

A = 200;         % Initial concentration of Source "S" and Initiator "I"
R = 200;         % Initial concentration of Reporter "R"
kt0 = 9E-7;      % Fwd reaction rate of Stage 1
kt1 = 2E-3;      % Fwd reaction rate of Stage 2    
kr = 1E-7;       % Reverse reaction rate of Stages 1&2&3
c = 0.001;       % Degree of complementarity for Stage 3
t_delay = 3300;  % Desired time delay   

tspan = [0 4*t_delay];      % Time interval over which we will simulate the DNA timer

Del_array = [];

for i=1:1:3000              % the range that the value of D can take
    dummy = 0.05*A+(i-1)*0.9*A/(3000-1);  % within 5%-95% of S/I - estimated to be sufficient
    Del_array = [Del_array dummy];
end

% ODE model of the nonlinear system. The states are as follows:
% y(1) = S, y(2) = I, y(3) = O, y(4) = W1, y(5) = D
% y(6) = W2, y(7) = W3, y(8) = R, y(9) = Q, y(10) = Fl
    
f = @(t,y) [-kt0*y(1)*y(2) + kr*y(3)*y(4); ... 
            -kt0*y(1)*y(2) + kr*y(3)*y(4); ...
             kt0*y(1)*y(2) - kr*y(3)*y(4) - kt1*y(3)*y(5) + kr*y(6)*y(7) - kt1*c*y(3)*y(8) + kr*y(9)*y(10); ...
             kt0*y(1)*y(2) - kr*y(3)*y(4); ...
             - kt1*y(3)*y(5) + kr*y(6)*y(7); ...
             kt1*y(5)*y(3) - kr*y(6)*y(7); ...
             kt1*y(5)*y(3) - kr*y(6)*y(7); ...
             -kt1*c*y(8)*y(3) + kr*y(9)*y(10); ...
             kt1*c*y(8)*y(3) - kr*y(9)*y(10); ...
             kt1*c*y(8)*y(3) - kr*y(9)*y(10)];

for i=1:1:3000
    
    % Initial conditions
    Del = Del_array(i);
    yinit = [A, A, 0, 0, Del, 0, 0, R, 0, 0];

    % Solve the nonliner ODEs
    [T,States] = ode45(f, tspan, yinit);

    % Find "t=t_delay" at which "D" becomes small enough
    
    dummy1 = States(:,5);     % Value of "D" at different time instants  
    dummy2 = States(:,10);    % Value of "F" at different time instants
    
    [index1, index2] = size(T);   % to ensure that ode45 runs for the same number of instances every time
    
        for j=1:1:index1                   
            if (abs(T(j)-t_delay)<20) % making sure that we are observing the output (Fl) at the time instant T which is close enough to t_delay 
                index = j;
                break 
            end
        end
        
        if (abs(dummy1(index)-dummy2(index))<1e-2*Del)   % abs(D - F) < 0.01*Del , where Del = [D]_0
            confirmed_D = Del;
            i=t_delay+1;
        end
end

Str1 = sprintf("The parameters supplied by you were I=S=R=%d and t_delay=%d.", A, t_delay);

 yinit0 = [A, A, 0, 0, confirmed_D, 0, 0, R, 0, 0];
 [t,y] = ode45(f, tspan, yinit0);
 func = y(:,10);

 % Setting min and max limits for the linearization interval - change if needed
 if (t_delay/A<=0.7)                    % for small values of t_delay - slower systems
     minX = 1.8*t_delay;
     maxX = 2.8*t_delay;
 elseif (t_delay/A>0.7)&&(t_delay/A<=1) % for slighly larger values of t_delay - faster systems
     minX = 1.6*t_delay;
     maxX = 2.3*t_delay;
 elseif (t_delay/A>1)&&(t_delay/A<=1.8) % for larger values of t_delay - even faster systems
     minX = 1.2*t_delay;
     maxX = 1.8*t_delay;
 elseif (t_delay/A>1.8)&&(A<=600)       % for large values of t_delay where A is not too large
     minX = 1.4*t_delay;
     maxX = 1.8*t_delay;    
 else                                   % for large values of t_delay with large A - the fastest systems
     minX = 1.2*t_delay;
     maxX = 1.6*t_delay;
 end

 % Fitting a line to the slope in order to determine the obtained t_delay
 index = (t>=minX)&(t<=maxX); % takes the points where you want to calculate your slope
 coefficients = polyfit(t(index),func(index), 1);
 yFitted = polyval(coefficients, t(index));

 % Finding an intersect of fitted line with X axis
 minY = min(yFitted);
 maxY = max(yFitted);
 xInt = maxX - (maxY*(maxX - minX))/(maxY - minY); % this is the calculated t_delay

 % Computational error
 Er = (xInt - t_delay)/t_delay;

if (Er <= 0.03) && (Er >= -0.03)     % if the error fits 3% margin
    yinit0 = [A, A, 0, 0, confirmed_D, 0, 0, R, 0, 0];
    [t,y] = ode45(f, tspan, yinit0);
    func = y(:,10);
    Str2 = sprintf("I recommend that the initial value of D is %0.2f nM", confirmed_D); 
end

while Er > 0.03 % if the error is > 3% (computed t_delay is too large)
      confirmed_D = confirmed_D*0.97; % reduce Del by 3% and recompute the system again
      yinit1 = [A, A, 0, 0, confirmed_D, 0, 0, R, 0, 0];
      [t,y] = ode45(f, tspan, yinit1);
      func = y(:,10);

      % Fitting a line to the slope in order to determine the obtained t_delay
      index = (t>=minX)&(t<=maxX); % takes the points where you want to calculate your slope
      coefficients = polyfit(t(index),func(index), 1);
      yFitted = polyval(coefficients, t(index));
 
      % Finding an intersect of fitted line with X axis
      maxY = max(yFitted);
      minY = min(yFitted);
      xInt = maxX - (maxY*(maxX - minX))/(maxY - minY); % intersect with X axis = t_delay
      Er = (xInt - t_delay)/t_delay;
      
          if (Er <= 0.03) && (Er >= -0.03) % break condition
              Str2 = sprintf("I recommend that the initial value of D is %0.2f nM", confirmed_D);
              break;
          end
end

while Er < -0.03  % if the error is < -3% (computed t_delay is too small)
     confirmed_D = confirmed_D*1.02; % increase Del by 2% and recompute the system again
     yinit2 = [A, A, 0, 0, confirmed_D, 0, 0, R, 0, 0];
     [t,y] = ode45(f, tspan, yinit2);
     func = y(:,10);

     % Fitting a line to the slope in order to determine the obtained t_delay
     index = (t>=minX)&(t<=maxX); % takes the points where you want to calculate your slope
     coefficients = polyfit(t(index),func(index), 1);
     yFitted = polyval(coefficients, t(index));
    
     % Finding an intersect of fitted line with X axis
     minY = min(yFitted);
     maxY = max(yFitted);
     xInt = maxX - (maxY*(maxX - minX))/(maxY - minY); % intersect with X axis = t_delay
     Er = (xInt - t_delay)/t_delay;
    
         if (Er <= 0.03) && (Er >= -0.03) % break condition
             Str2 = sprintf("I recommend that the initial value of D is %0.2f nM", confirmed_D);
             break
         end
end

% Plotting the final solution
    p1 = plot(t,y(:,10));
    p1.LineWidth = 2;
    xlabel('Time')
    ylabel('Concentration')
    title('Programmable t_{delay} for TMT circuit')
    grid

% Plotting the fitted line
    hold on
    p2 = plot(t(index),yFitted);
    p2.LineWidth = 2;

% Plotting the intersect with X-axis
    hold on
    p3 = plot(xInt,0,'m*','MarkerSize', 10);
    h = [p1;p2;p3];
    legend(h,'Fluorophore', "Fitted line", "Calculated t_{delay}");
   
% Showing the value of X-intersect on the plot
    strValues = strtrim(cellstr(num2str(xInt,"%0.0f")));
    text(xInt,10,strValues,"VerticalAlignment","bottom","HorizontalAlignment","center");
    hold off

disp(Str1)
disp(Str2)

